package sun.java2d.pipe;

import java.awt.Rectangle;
import sun.font.GlyphList;
import sun.java2d.SunGraphics2D;

public class TextRenderer extends GlyphListPipe
{
    CompositePipe outpipe;
    
    public TextRenderer(CompositePipe outpipe) {
        this.outpipe = outpipe;
    }
    
    @Override
    protected void drawGlyphList(SunGraphics2D sunGraphics2D, GlyphList list) {
        int numGlyphs = list.getNumGlyphs();
        Region compClip = sunGraphics2D.getCompClip();
        int loX = compClip.getLoX();
        int loY = compClip.getLoY();
        int hiX = compClip.getHiX();
        int hiY = compClip.getHiY();
        Object startSequence = null;
        try {
            int[] bounds = list.getBounds();
            Rectangle s = new Rectangle(bounds[0], bounds[1], bounds[2] - bounds[0], bounds[3] - bounds[1]);
            startSequence = this.outpipe.startSequence(sunGraphics2D, sunGraphics2D.untransformShape(s), s, bounds);
            for (int i = 0; i < numGlyphs; ++i) {
                list.setGlyphIndex(i);
                int[] metrics = list.getMetrics();
                int n = metrics[0];
                int n2 = metrics[1];
                int n3 = metrics[2];
                int n4 = n + n3;
                int n5 = n2 + metrics[3];
                int n6 = 0;
                if (n < loX) {
                    n6 = loX - n;
                    n = loX;
                }
                if (n2 < loY) {
                    n6 += (loY - n2) * n3;
                    n2 = loY;
                }
                if (n4 > hiX) {
                    n4 = hiX;
                }
                if (n5 > hiY) {
                    n5 = hiY;
                }
                if (n4 > n && n5 > n2 && this.outpipe.needTile(startSequence, n, n2, n4 - n, n5 - n2)) {
                    this.outpipe.renderPathTile(startSequence, list.getGrayBits(), n6, n3, n, n2, n4 - n, n5 - n2);
                }
                else {
                    this.outpipe.skipTile(startSequence, n, n2);
                }
            }
        }
        finally {
            if (startSequence != null) {
                this.outpipe.endSequence(startSequence);
            }
        }
    }
}
